#!/bin/bash

HELP_INFO='''
# Before Usage:
# FIPS_MODE environment variable needs to be set to one of the following disabled/enabled/strict/strong
# More info for each mode can be found here - https://confluence.eng.vmware.com/display/CAEng/FIPS+Implementation+Notes
#
# Usage:
# source /usr/local/bin/setup_security.sh
# or
# source /usr/local/bin/setup_security.sh --external-cert-path=<SOME_PATH>
#
# Default --enternal-cert-path - /run/external-certs
#
# After that the following environment variables will be available for use
# * TRUST_STORE_PASSWORD - *deprecated* - contains the generated truststore password, will not be
#   changed if passed from outside. Will be removed in later versions.
# * JAVA_OPTS - will append the options needed to enable BC FIPS.
# * BCFIPS_JARS - the full path to bc-fips.jar or jars separated with :, so this can be added to
#   the start of application classpath like
#        java -cp ${BCFIPS_JARS:+"$BCFIPS_JARS":}$org_class_path
#
# The following files are located in the directory pointed by LIB_SECURITY_PATH
# * cacerts.pass - contains the password for the truststore in a format accepted by the Java keytool
'''

function print_help() {
  cat <<EOF
$HELP_INFO
EOF
}

JAVA_FIPS_SECURITY_OPTS=""

JAVA_SECURITY_PATH=${JAVA_SECURITY_PATH:-$JAVA_HOME/conf/security}

DEFAULT_JAVA_SECURITY_FILE="$JAVA_SECURITY_PATH/java.security"

FIPS_MODE_DISABLED="disabled"
FIPS_MODE_ENABLED="enabled"
FIPS_MODE_STRICT="strict"
FIPS_MODE_STRONG="strong"

# set default BCFIPS_PROVIDER_PARAMS if not externally provided
if test -z "$BCFIPS_PROVIDER_PARAMS"
then
  BCFIPS_PROVIDER_PARAMS='C:DEFRND[HMACSHA512(256)];HYBRID;ENABLE{ALL};'
fi


FIPS_SECURITY_PROVIDERS='''security.provider.1=org.bouncycastle.jcajce.provider.BouncyCastleFipsProvider '''"$BCFIPS_PROVIDER_PARAMS"'''
security.provider.2=org.bouncycastle.jsse.provider.BouncyCastleJsseProvider fips:BCFIPS'''


if [ $# -eq 1 ] && [ "$1" = "-h"  -o "$1" = "--help" ]
then
  print_help
  exit 0
fi

# Defaults
EXTERNAL_CERT_PATH=${EXTERNAL_CERT_PATH:-/etc/ssl/certs}
printf '%s\n' "EXTERNAL_CERT_PATH=$EXTERNAL_CERT_PATH"

#-----------------------------  ADDITIONAL -------------------------------------
curl -OsSL "https://s3.amazonaws.com/rds-downloads/rds-combined-ca-bundle.pem" --output-dir $EXTERNAL_CERT_PATH
printf '%s\n' "Saved RDS Combined Certificate Bundle to $EXTERNAL_CERT_PATH"

curl -OsSL "https://truststore.pki.us-gov-west-1.rds.amazonaws.com/global/global-bundle.pem" --output-dir $EXTERNAL_CERT_PATH
printf '%s\n' "Saved GovCloud(us-gov-west-1) RDS Combined Certificate Bundle to $EXTERNAL_CERT_PATH"

pushd $EXTERNAL_CERT_PATH
awk 'BEGIN {c=0;} /BEGIN CERT/{c++} { print > "rds-cert-" c ".pem"}' < "$EXTERNAL_CERT_PATH/rds-combined-ca-bundle.pem"
awk 'BEGIN {c=0;} /BEGIN CERT/{c++} { print > "us-gov-west-1-rds-cert-" c ".pem"}' < "$EXTERNAL_CERT_PATH/global-bundle.pem"
popd

rm "$EXTERNAL_CERT_PATH/rds-combined-ca-bundle.pem"
rm "$EXTERNAL_CERT_PATH/global-bundle.pem"
#-------------------------------  OPTIONS --------------------------------------
if [ ! -n "$FIPS_MODE" ]
then
  export FIPS_MODE="$FIPS_MODE_DISABLED"
  printf '%s\n' "WARNING: FIPS_MODE environment variable is not set! The default value is set to $FIPS_MODE"
else
  export FIPS_MODE=${FIPS_MODE,,}
fi
printf '%s\n' "FIPS_MODE=$FIPS_MODE"

if ! [[ "$FIPS_MODE" =~ ^($FIPS_MODE_DISABLED|$FIPS_MODE_ENABLED|$FIPS_MODE_STRICT|$FIPS_MODE_STRONG)$ ]]
then
  printf '%s\n' "ERROR: FIPS_MODE environment variable ($FIPS_MODE) is not set correctly! Use $FIPS_MODE_DISABLED/$FIPS_MODE_ENABLED/$FIPS_MODE_STRICT/$FIPS_MODE_STRONG"
  exit 2
fi

#--------------------------------- CHANGE PASSWORD -----------------------------
LIB_SECURITY_PATH="${LIB_SECURITY_PATH:-$JAVA_HOME/lib/security}"
if test -e "$LIB_SECURITY_PATH/cacerts"
then
  printf '%s\n' "$LIB_SECURITY_PATH/cacerts already exists - will not overwrite from security-template"
else
  cp -Rp $LIB_SECURITY_PATH/security-template/* $LIB_SECURITY_PATH || { echo "Cannot find cacerts file in $JAVA_HOME" ; exit 1 ; }
fi
printf '%s\n' "Found cacerts in $LIB_SECURITY_PATH"
ls -al "$LIB_SECURITY_PATH"

CACERTS_PATH="$LIB_SECURITY_PATH/cacerts"

if test -n "$TRUST_STORE_PASSWORD"; then
  keytool -storepasswd -keystore "$CACERTS_PATH" -storepass changeit -new:env TRUST_STORE_PASSWORD
else
  printf '%s/n' "TRUST_STORE_PASSWORD env variable not found"
fi
#-------------------------------IMPORT CERTIFICATES-----------------------------
printf '%s\n' "Using external cert path - $EXTERNAL_CERT_PATH"

for cert in "$EXTERNAL_CERT_PATH"/*.pem
do
  if test -f "$cert"
  then
    printf '%s\n' "Importing $cert"
    keytool -import -trustcacerts -keystore "$CACERTS_PATH" \
        -storepass:env TRUST_STORE_PASSWORD \
        -noprompt \
        -alias "prelude-int-$(basename $cert)" \
        -file "$cert"
  fi
done

#-------------------------------SETUP FIPS--------------------------------------
# Define JAVA_OPTS if it is not yet defined
JAVA_OPTS=${JAVA_OPTS:-}

if [[ "$FIPS_MODE" =~ ^($FIPS_MODE_ENABLED|$FIPS_MODE_STRICT|$FIPS_MODE_STRONG)$ ]]
then

  OVERRIDE_JAVA_SECURITY_FILE="$LIB_SECURITY_PATH/vmware-override-java.security"
  TMP_FILE="$LIB_SECURITY_PATH/tmp_file"
  TMP_FILE_BAK="$LIB_SECURITY_PATH/tmp_file.bak"

  printf '%s\n' "Setup $OVERRIDE_JAVA_SECURITY_FILE with bouncycastle security providers..."
  # Create vmware-override-java.security file based on default java.security
  cp $DEFAULT_JAVA_SECURITY_FILE $OVERRIDE_JAVA_SECURITY_FILE
  # Remove all lines containing security.provider.<any number>
  sed -i.bak '/security.provider.[0-9]/d' $OVERRIDE_JAVA_SECURITY_FILE
  sed -i.bak 's/SunX509/X.509/' $OVERRIDE_JAVA_SECURITY_FILE
  # Add FIPS list of providers with their preference order
  awk -v security_providers="$FIPS_SECURITY_PROVIDERS" '{print} /# List of providers and their preference orders.*/{print security_providers}' $OVERRIDE_JAVA_SECURITY_FILE > $TMP_FILE

  # set default keystore type to bcfks
  printf '%s\n' "Set keystore.type to BCFKS"
  grep -q '^\s*keystore\.type\s*=' $TMP_FILE || echo 'keystore.type=bcfks' >> $TMP_FILE
  sed 's/\s*keystore\.type\s*=.*$/keystore.type=bcfks/' $TMP_FILE > $TMP_FILE_BAK
  cp $TMP_FILE_BAK $TMP_FILE

  printf '%s\n' "Set securerandom.strongAlgorithms to Java default one"
  grep -q '^\s*securerandom\.strongAlgorithms\s*=' $TMP_FILE || echo 'securerandom.strongAlgorithms=NativePRNGBlocking:SUN' >> $TMP_FILE
  sed 's/^\(securerandom\.strongAlgorithms\s*=\s*\).*$/\1NativePRNGBlocking:SUN/' $TMP_FILE > $TMP_FILE_BAK
  cp $TMP_FILE_BAK $TMP_FILE
  cat $TMP_FILE > $OVERRIDE_JAVA_SECURITY_FILE

  printf '%s\n' "Defined BCFIPS_JARS='$BCFIPS_JARS' to be added java class path."

#-------------------------------CREATE BC STORE-----------------------------------------
  BCFKS_TRUSTSTORE="$LIB_SECURITY_PATH/truststore.bcfks"

  keytool -importkeystore -srckeystore $CACERTS_PATH -destkeystore $BCFKS_TRUSTSTORE \
    -noprompt \
    -srcstoretype JKS -deststoretype BCFKS \
    -srcstorepass:env TRUST_STORE_PASSWORD -deststorepass:env TRUST_STORE_PASSWORD \
    -provider org.bouncycastle.jcajce.provider.BouncyCastleFipsProvider \
    -providerpath "$BCFIPS_JARS" \
    -provider org.bouncycastle.jsse.provider.BouncyCastleJsseProvider \
    -providerpath "$BCFIPS_JARS"

#-------------------------------Create java-fips executable-----------------------------------------

  cat <<-EOF > "$JAVA_HOME/bin/java-fips"
#!/bin/sh
exec java \$JAVA_FIPS_OPTS \$JAVA_FIPS_CLASSPATH:\${JAVA_CLASSPATH:-.} \$@
EOF
  chmod +x "$JAVA_HOME/bin/java-fips"

  printf '%s\n' "Created 'java-fips' alias under $JAVA_HOME/bin/java-fips with embedded JAVA_FIPS_SECURITY_OPTS defined as follows: '$JAVA_FIPS_SECURITY_OPTS $JAVA_FIPS_SECURITY_CLASSPATH'"

fi
#----------------------------------------------------------------------------------
